/*---------------------------------------------------------------------------\
| Project:  PIC18F458 CAN Node Core Firmware
|
| File:     handlers.c
|
| Descr.:   Application specific code which must be executed on an interrupt.
|           Essentially, these are "hooks" into the core's interrupt service
|           routines.
|
|           YOU SHOULD MODIFIY THIS FILE FOR YOUR APPLICATION, ALONG WITH
|           THE CONFIG.H FILE. FEEL FREE TO EVEN CUT AND COPY THESE FUNCTIONS
|           TO OTHER FILES.
|
|           HOWEVER, DO *NOT* CHANGE THE #IF, OR THE NAME AND TYPE OF THE
|           HANDLER FUNCTIONS.
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 09/07/2002 | Created to store examples of handlers.
\---------------------------------------------------------------------------*/

#include ".\piccore\core.h"
#include "imu.h"

/*---------------------------------------------------------------------------\
| Function:     ADCApplicationHandler
|
| Purpose:      Set the temp_go flag when pic adc is done its conversion
|
| Discussion:   See purpose
|
| Expects:      The ADON=1, GODONE previously asserted.
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 09/12/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

#if INTS_ADC_APPLICATION_HOOK == INTS_HOOK_ENABLED

void ADCApplicationHandler (void)
{
    temp_go = 1;
}

#endif

// I commented out the line below this to get rid of an
// linking error ASK ANDREW WHAT I DID.
 #if INTS_TIMER2_APPLICATION_HOOK == INTS_HOOK_ENABLED

/*---------------------------------------------------------------------------\
| Function:     Application_Timer2_Handler
|
| Purpose:      Set the correct bits in the Sample_ADC
|
| Discussion:   By compairing the global vars of sample rates against
|               Those of the local sample rate count.  Integer divide
|               by count not mod.
|
| Expects:      The four global sample rate variables to be set.
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 09/07/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

void Timer2ApplicationHandler()
{

    static uint8_t acc_x_sample_divisor_Counter = 1;
    static uint8_t acc_y_sample_divisor_Counter = 1;
    static uint8_t acc_z_sample_divisor_Counter = 1;
    static uint8_t acc_q_sample_divisor_Counter = 1;
    static uint8_t gyro_p_sample_divisor_Counter = 1;
    static uint8_t gyro_y_sample_divisor_Counter = 1;
    static uint8_t gyro_r_sample_divisor_Counter = 1;
    static uint8_t temp_sample_divisor_Counter = 1;
    static uint8_t pres_sample_divisor_Counter = 1;

//Start to set sensor flags
    //set flags fro accelorometers
    if(acc_x_sample_divisor_Counter >= acc_x_sample_divisor)
    {
        acc_x_flag = TRUE;
        acc_x_sample_divisor_Counter = 1;
    } else {
        ++acc_x_sample_divisor_Counter;
    }

    if(acc_y_sample_divisor_Counter >= acc_y_sample_divisor)
    {
        acc_y_flag = TRUE;
        acc_y_sample_divisor_Counter = 1;
    } else {
        ++acc_y_sample_divisor_Counter;
    }

    if(acc_z_sample_divisor_Counter >= acc_z_sample_divisor)
    {
        acc_z_flag = TRUE;
        acc_z_sample_divisor_Counter = 1;
    } else {

        ++acc_z_sample_divisor_Counter;
    }

    if(acc_q_sample_divisor_Counter >= acc_q_sample_divisor)
    {
        acc_q_flag = TRUE;
        acc_q_sample_divisor_Counter = 1;
    } else {
        ++acc_q_sample_divisor_Counter;
    }

    //set flags for gyroscope
    if(gyro_p_sample_divisor_Counter >= gyro_p_sample_divisor)
    {
        gyro_p_sample_divisor_Counter = 1;
        gyro_p_flag = TRUE;
    } else {
        ++gyro_p_sample_divisor_Counter;
    }

    if(gyro_y_sample_divisor_Counter >= gyro_y_sample_divisor)
    {
        gyro_y_sample_divisor_Counter = 1;
        gyro_y_flag = TRUE;
    } else {
        ++gyro_y_sample_divisor_Counter;
    }

    if(gyro_r_sample_divisor_Counter >= gyro_r_sample_divisor)
    {
        gyro_r_sample_divisor_Counter = 1;
        gyro_r_flag = TRUE;
    } else {
        ++gyro_r_sample_divisor_Counter;
    }

    //set pressure sensor flag
    if(pres_sample_divisor_Counter >= pres_sample_divisor)
    {
        pres_flag = TRUE;
        pres_sample_divisor_Counter = 1;
    } else {
        ++pres_sample_divisor_Counter;
    }

    //set temperature sensor flag
    if(temp_sample_divisor_Counter >= temp_sample_divisor)
    {
        temp_flag = TRUE;
        temp_sample_divisor_Counter = 1;
    } else {
        ++temp_sample_divisor_Counter;
    }
//end sensor flag setting


// Start taking first ADC sample the rest will be taken care of
// by Int0_Handler subroutine.  Because The temperature data is not
// converted with the Max197 it needs to be handeled with the
// 18f458's internal ADC.

    if(acc_x_flag)
    {
      max197_cmd = 0b01000010;
      LATD = max197_cmd;
      WR_ = 0;
      WR_ = 1;
    }

    if(acc_y_flag)
    {
      max197_cmd = 0b01000111;
      LATD = max197_cmd;
      WR_ = 0;
      WR_ = 1;
    } else if(acc_z_flag) {
      max197_cmd = 0b01000000;
      LATD = max197_cmd;
      WR_ = 0;
      WR_ = 1;
    } else if (acc_q_flag) {
      max197_cmd = 0b01000011;
      LATD = max197_cmd;
      WR_ = 0;
      WR_ = 1;
    } else if(gyro_p_flag) {
      max197_cmd = 0b01000100;
      LATD = max197_cmd;
      WR_ = 0;
      WR_ = 1;
    } else if(gyro_y_flag) {
      max197_cmd = 0b01000101;
      LATD = max197_cmd;
      WR_ = 0;
      WR_ = 1;
    }else if(gyro_r_flag) {
      max197_cmd = 0b01000001;
      LATD = max197_cmd;
      WR_ = 0;
      WR_ = 1;
    } else if(pres_flag) {
      max197_cmd = 0b01000110;
      LATD = max197_cmd;
      WR_ = 0;
      WR_ = 1;
    }

    if(temp_flag)
    {
      GODONE = 1;
      temp_flag = 0;
    }

} // end Application_Timer2_Handler

#endif

#if INTS_INT0_APPLICATION_HOOK == INTS_HOOK_ENABLED

/*---------------------------------------------------------------------------\
| Function: Int0_Interupt
|
| Purpose:  Take in and process data from the MAX197, clear flags as they GODONE.
|
| Discussion:  It is very IMPORTANT that this makes its calls to the
|              MAX197 in the same order as the Application_Timer2_Handler
|              NOTE:That temperature AD conversion is handeled by the
|                   18f458's internal ADC.
|
| Expects:  Global flags for each sensor that needs to be sampled, at least
|           one of then needs to have been sampled before this function
|           would be called.
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 09/07/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

void Int0ApplicationHandler ()
{
 if(acc_x_flag)
 {
     HBEN = 0;  // don't assume anything
     TRISD = 0xff; //switch to input mode
     RD_ = 0; //assert the read pin
     acc_x_adc_result_l = PORTD; //Read in the first low byte
     HBEN = 1; //asser the High Byte Enable pin.
     acc_x_adc_result_h = PORTD;
     RD_ = 1; // Deassert the read pin
     TRISD = 0x00; //Deassert the
     acc_x_flag = 0; //Clear this sensor flag
     if( ! (acc_y_flag | acc_z_flag | acc_q_flag) )
     {
       acc_go = 1;
     }

     //look for and start next
     if(acc_y_flag)
     {
        max197_cmd = 0b01000111;
        LATD = max197_cmd;
        WR_ = 0;
        WR_ = 1;
     } else if(acc_z_flag) {
        max197_cmd = 0b01000000;
        LATD = max197_cmd;
        WR_ = 0;
        WR_ = 1;
     } else if(acc_q_flag) {
        max197_cmd = 0b01000011;
        LATD = max197_cmd;
        WR_ = 0;
        WR_ = 1;
     } else if(gyro_p_flag) {
        max197_cmd = 0b01000100;
        LATD = max197_cmd;
        WR_ = 0;
        WR_ = 1;
     } else if(gyro_y_flag) {
        max197_cmd = 0b01000101;
        LATD = max197_cmd;
        WR_ = 0;
        WR_ = 1;
     } else if(gyro_r_flag) {
        max197_cmd = 0b01000001;
        LATD = max197_cmd;
        WR_ = 0;
        WR_ = 1;
     } else if(pres_flag) {
        max197_cmd = 0b01000110;
        LATD = max197_cmd;
        WR_ = 0;
        WR_ = 1;
     }
     return;
 } // end check for acc_x_flag

 if(acc_y_flag)
 {
    HBEN = 0;                  //deassert High Byte Enable pin
    TRISD = 0xff;              //switch to input mode
    RD_ = 0;                   //assert the read pin
    acc_y_adc_result_l = PORTD; //Read in the first low byte
    HBEN = 1;                  //asser the High Byte Enable pin.
    acc_y_adc_result_h = PORTD; //Read in High byte
    RD_ = 1;                   // Deassert the read pin
    TRISD = 0x00;              //Deassert the
    acc_y_flag = 0;             //Clear this sensor flag
    if( ! (acc_z_flag | acc_q_flag) )
    {
      acc_go =1;
    }
    if(acc_z_flag)
    {
        max197_cmd = 0b01000000;
        LATD = max197_cmd;
        WR_ = 0;
        WR_ = 1;
    } else if(acc_q_flag) {
        max197_cmd = 0b01000011;
        LATD = max197_cmd;
        WR_ = 0;
        WR_ = 1;
    } else if(gyro_p_flag) {
        max197_cmd = 0b01000100;
        LATD = max197_cmd;
        WR_ = 0;
        WR_ = 1;
    } else if(gyro_y_flag) {
        max197_cmd = 0b01000101;
        LATD = max197_cmd;
        WR_ = 0;
        WR_ = 1;
    } else if(gyro_r_flag) {
        max197_cmd = 0b01000001;
        LATD = max197_cmd;
        WR_ = 0;
        WR_ = 1;
    } else if(pres_flag) {
        max197_cmd = 0b01000110;
        LATD = max197_cmd;
        WR_ = 0;
        WR_ = 1;
    }
    return;
 }

 if(acc_z_flag)
 {
    HBEN = 0; //deassert High Byte Enable pin
    TRISD = 0xff; //switch to input mode
    RD_ = 0; //assert the read pin
    acc_z_adc_result_l = PORTD; //Read in the first low byte
    HBEN = 1; //asser the High Byte Enable pin.
    acc_z_adc_result_h = PORTD; // special read thingy
    RD_ = 1; // Deassert the read pin
    TRISD = 0x00; //Deassert the
    acc_z_flag = 0; //Clear this sensor flag
    if( ! (acc_q_flag) )
    {
      acc_go =1;
    }

    // check to see if any other sensors need to be sampeled
    if(acc_q_flag)
    {
        max197_cmd = 0b01000011;
        LATD = max197_cmd;
        WR_ = 0;
        WR_ = 1;
    } else if(gyro_p_flag) {
        max197_cmd = 0b01000100;
        LATD = max197_cmd;
        WR_ = 0;
        WR_ = 1;
    } else if(gyro_y_flag) {
        max197_cmd = 0b01000101;
        LATD = max197_cmd;
        WR_ = 0;
        WR_ = 1;
    } else if(gyro_r_flag) {
        max197_cmd = 0b01000001;
        LATD = max197_cmd;
        WR_ = 0;
        WR_ = 1;
    } else if(pres_flag) {
        max197_cmd = 0b01000110;
        LATD = max197_cmd;
        WR_ = 0;
        WR_ = 1;
    }
    return;
 }

 if(acc_q_flag){
    HBEN = 0; //deassert High Byte Enable pin
    TRISD = 0xff; //switch to input mode
    RD_ = 0; //assert the read pin
    acc_q_adc_result_l = PORTD; //Read in the first low byte
    HBEN = 1; //asser the High Byte Enable pin.
    acc_q_adc_result_h = PORTD; // special read thingy
    RD_ = 1; // Deassert the read pin
    TRISD = 0x00; //Deassert the
    acc_q_flag = 0; //Clear this sensor flag
    acc_go = 1;

    // check to see if any other sensors need to be sampeled
    if(gyro_p_flag)
    {
        max197_cmd = 0b01000100;
        LATD = max197_cmd;
        WR_ = 0;
        WR_ = 1;
    } else if(gyro_y_flag) {
        max197_cmd = 0b01000101;
        LATD = max197_cmd;
        WR_ = 0;
        WR_ = 1;
    } else if(gyro_r_flag) {
        max197_cmd = 0b01000001;
        LATD = max197_cmd;
        WR_ = 0;
        WR_ = 1;
    } else if(pres_flag) {
        max197_cmd = 0b01000110;
        LATD = max197_cmd;
        WR_ = 0;
        WR_ = 1;
    }
    return;
 }

 if(gyro_p_flag)
 {
    HBEN = 0; //deassert High Byte Enable pin
    TRISD = 0xff; //switch to input mode
    RD_ = 0; //assert the read pin
    gyro_p_adc_result_l = PORTD; //Read in the first low byte
    HBEN = 1; //asser the High Byte Enable pin.
    gyro_p_adc_result_h = PORTD; // special read thingy
    RD_ = 1; // Deassert the read pin
    TRISD = 0x00; //Deassert the
    gyro_p_flag = 0; //Clear this sensor flag
    if(! (gyro_y_flag | gyro_r_flag))
    {
        gyro_go = 1;
    }
    // check to see if any other sensors need to be sampeled
    if(gyro_y_flag)
    {
        max197_cmd = 0b01000101;
        LATD = max197_cmd;
        WR_ = 0;
        WR_ = 1;
    } else if(gyro_r_flag) {
        max197_cmd = 0b01000001;
        LATD = max197_cmd;
        WR_ = 0;
        WR_ = 1;
    } else if(pres_flag) {
        max197_cmd = 0b01000110;
        LATD = max197_cmd;
        WR_ = 0;
        WR_ = 1;
    }
    return;
 }

 if(gyro_y_flag)
 {
     HBEN = 0; //deassert High Byte Enable pin
     TRISD = 0xff; //switch to input mode
     RD_ = 0; //assert the read pin
     gyro_y_adc_result_l = PORTD; //Read in the first low byte
     HBEN = 1; //asser the High Byte Enable pin.
     gyro_y_adc_result_h = PORTD;
     RD_ = 1; // Deassert the read pin
     TRISD = 0x00; //Deassert the
     gyro_y_flag = 0; //Clear this sensor flag
     if(! (gyro_r_flag))
     {
        gyro_go = 1;
     }
     // check to see if any other sensors need to be sampeled
     if(gyro_r_flag) {
        max197_cmd = 0b01000001;
        LATD = max197_cmd;
        WR_ = 0;
        WR_ = 1;
     } else if(pres_flag) {
        max197_cmd = 0b01000110;
        LATD = max197_cmd;
        WR_ = 0;
        WR_ = 1;

     }
     return;
 }

 if(gyro_r_flag)
 {
     HBEN = 0; //deassert High Byte Enable pin
     TRISD = 0xff; //switch to input mode
     RD_ = 0; //assert the read pin
     gyro_r_adc_result_l = PORTD; //Read in the first low byte
     HBEN = 1; //asser the High Byte Enable pin.
     gyro_r_adc_result_h = PORTD; // special read thingy
     RD_ = 1; // Deassert the read pin
     TRISD = 0x00; //Deassert the
     gyro_r_flag = 0; //Clear this sensor flag
     gyro_go = 1;
     // check to see if any other sensors need to be sampeled
     if(pres_flag)
     {
         max197_cmd = 0b01000110;
         LATD = max197_cmd;
         WR_ = 0;
         WR_ = 1;
     }
     return;
 }

 if(pres_flag)
 {
     TRISD = 0xff; //switch to input mode
     RD_ = 0;      //assert the read pin
     pres_adc_result_l = PORTD; //Read in the first low byte
     HBEN = 1;               //asser the High Byte Enable pin.
     pres_adc_result_h = PORTD; // special read thingy
     HBEN = 0; //deassert High Byte Enable pin
     RD_ = 1; // Deassert the read pin
     TRISD = 0x00; //Deassert the
     pres_flag = 0; //Clear this sensor flag
     pres_go = 1;
 }


} // end Int0_Interupt

#endif

