/*---------------------------------------------------------------------------\
| Project:          Generic LV2 PIC18F458 CAN node firmware
|
| File:             main.c
|
| Description:      Mainline routines.
|
| Global Routines:  Main()
|
| Local Routines:   None
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 06/01/2002 | Stolen from ADG code.
\---------------------------------------------------------------------------*/



//------------------------------------------------------------------------
//  #includes
//------------------------------------------------------------------------

#include ".\piccore\core.h"   // Includes for the project (includes other includes!)

#if CORE_VERSION != 4
#error HEY! The core version has changed! Make SURE your interfaces match!
#endif

#include "imu.h"   // Includes for the imu firmware program

//------------------------------------------------------------------------
// Global variable declarations
//------------------------------------------------------------------------
//These globals are used to define the sample rates for IMU and Temp and Pressure
uint8_t acc_x_sample_divisor = 1; //Default rate of 2.5kHz (w/timer2@2.5kHz)
uint8_t acc_y_sample_divisor = 1; //Default rate of 2.5kHz (w/timer2@2.5kHz)
uint8_t acc_z_sample_divisor = 1; //Default rate of 2.5kHz (w/timer2@2.5kHz)
uint8_t acc_q_sample_divisor = 1; //Default rate of 2.5kHz (w/timer2@2.5kHz)
uint8_t gyro_p_sample_divisor = 3;//Default rate of 833Hz (w/timer2@2.5kHz)
uint8_t gyro_y_sample_divisor = 3;//Default rate of 833Hz (w/timer2@2.5kHz)
uint8_t gyro_r_sample_divisor = 3;//Default rate of 833Hz (w/timer2@2.5kHz)
uint8_t pres_sample_divisor = 25;//Default rate of 100Hz (w/timer2@2.5kHz)
uint8_t temp_sample_divisor = 250;//Default rate of 10Hz (w/timer2@2.5kHz)

// if a sample needs to be taken its associated flag will be set to TRUE
bit acc_x_flag;
bit acc_y_flag;
bit acc_z_flag;
bit acc_q_flag;
bit gyro_p_flag;
bit gyro_y_flag;
bit gyro_r_flag;
bit pres_flag;
bit temp_flag;

// holds commands for Max197 AD chip, see data sheet for more details
uint8_t max197_cmd;

// hold results from the ADC before sending in the CAN message
uint8_t acc_x_adc_result_h;
uint8_t acc_x_adc_result_l;

uint8_t acc_y_adc_result_h;
uint8_t acc_y_adc_result_l;

uint8_t acc_z_adc_result_h;
uint8_t acc_z_adc_result_l;

uint8_t acc_q_adc_result_h;
uint8_t acc_q_adc_result_l;

uint8_t gyro_p_adc_result_h;
uint8_t gyro_p_adc_result_l;

uint8_t gyro_y_adc_result_h;
uint8_t gyro_y_adc_result_l;

uint8_t gyro_r_adc_result_h;
uint8_t gyro_r_adc_result_l;

uint8_t pres_adc_result_h;
uint8_t pres_adc_result_l;


// flags for when a can message is ready to be sent
bit acc_go;
bit gyro_go;
bit pres_go;
bit temp_go;

/*---------------------------------------------------------------------------\
| Function:     main
|
| Purpose:      Run all Mainline code
|
| Discussion:   Doesn't do much, huh?
|
| Expects:      Uh let's see... nothing.
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 02/10/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

void main (void)
{
    static CanMessage_t tempMsg;
    // static CanMessage_t  tempMsg2;

    // Initialize everything and its brother

    Initialize();


    // Loop for ever...and......ev....er....and.......ev............er.

    while (TRUE)
    {
        if(acc_go)
        {
            tempMsg.header.all = IMU_ACC_DATA_OUT;
            tempMsg.data[0] = acc_x_adc_result_h;
            tempMsg.data[1] = acc_x_adc_result_l;
            tempMsg.data[2] = acc_y_adc_result_h;
            tempMsg.data[3] = acc_y_adc_result_l;
            tempMsg.data[4] = acc_z_adc_result_h;
            tempMsg.data[5] = acc_z_adc_result_l;
            tempMsg.data[6] = acc_q_adc_result_h;
            tempMsg.data[7] = acc_q_adc_result_l;
            acc_go = 0;
            SendCanMessage(&tempMsg);
        }
        if(gyro_go)
        {
            tempMsg.header.all = IMU_GYRO_DATA_OUT;
            tempMsg.data[0] = gyro_p_adc_result_h;
            tempMsg.data[1] = gyro_p_adc_result_l;
            tempMsg.data[2] = gyro_y_adc_result_h;
            tempMsg.data[3] = gyro_y_adc_result_l;
            tempMsg.data[4] = gyro_r_adc_result_h;
            tempMsg.data[5] = gyro_r_adc_result_l;
            SendCanMessage(&tempMsg);
            gyro_go = 0;
        }
        if(pres_go)
        {
            tempMsg.header.all = PRES_DATA_OUT;
            tempMsg.data[0] = pres_adc_result_l;
            tempMsg.data[1] = pres_adc_result_h;
            SendCanMessage(&tempMsg);
            pres_go = 0;
        }
        if(temp_go)
        {
            tempMsg.header.all = TEMP_DATA_OUT;
            tempMsg.data[0] = ADRESL;
            tempMsg.data[1] = ADRESH; //expects ADFM=1 (pg234 458 datasheet)
            SendCanMessage(&tempMsg);
            temp_go = 0;
        }
        if(canMessageAvailable)
        {
            canMessageAvailable = 0; //reset to parse flag
            GetCanMessage(&tempMsg);
            Parse_Can_Message(&tempMsg);
        }
    }

}







/*---------------------------------------------------------------------------\
| Function:     Initialize
|
| Purpose:      Initialize everything. Really.
|
| Discussion:   Did I mention initialization?
|
| Expects:      To be called after a RESET - eg, registers reset to known
|               values and and interrupts off.
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 02/10/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

void Initialize (void)
{

    /*
        EVENTUALLY: Check for a reset: If there hasn't been a reset,
        then something has gone horribly wrong. Do a SOFTWARE RESET
        here to put everything in a known state.
    */

    // Setting up the CAN bus peripheral: We're passing constants
    // to the initialization routine so that we can just set the
    // initialize initialization :) here.

    const CanHeader_t canMaskB0 = (0x07FF << 5);
    const CanHeader_t canMaskB1 = (0x07FF << 5);
    const CanHeader_t CanFilterB0F0 = (0x07FC <<5);
    const CanHeader_t CanFilterB0F1 = (0x0201 << 5);
    const CanHeader_t CanFilterB1F2 = (0x0000 << 5);
    const CanHeader_t CanFilterB1F3 = (0x0000 << 5);
    const CanHeader_t CanFilterB1F4 = (0x0000 << 5);
    const CanHeader_t CanFilterB1F5 = (0x0000 << 5);

    // Initiaize the ADC for use by the Temp sensor.
    // Start by selecting the ADC channel on the PIC
        CHS0 = 0;
        CHS1 = 0;
        CHS2 = 0;
    // Next turn on the ADC
        ADON = 1;
        GODONE = 0;
    // Make the 6 most significant bits of ADRESH are 0's
        ADFM = 1;
    // Set ADC conversion clock select bits
    // this config insures Fosc/32
        ADCS2 = 0;
        ADCS1 = 0;
        ADCS0 = 0;

    // Get the outputs stabilized FIRST.
    // PORTA RA0 = IMU TEMP
    // PORTB RB3 = CAN RX RB2 = CAN TX RB0 = INT0
    // PORTC RC0 = /RD RC1 = /WR RC2 = TEST
    // PROTD = DATA BUS
    // PORTE RE0 = /CS RE2 = HBEN

    InitializePorts (  0b00000000, // latchA
                       0b00000001, // directionA (1 = in, 0 = out)
                       0b00000000, // latchB
                       0b00001001, // directionB (1 = in, 0 = out) (CAN!!)
                       0b00000011, // latchC
                       0b00000000, // directionC (1 = in, 0 = out)
                       0b00000000, // latchD
                       0b00000000, // directionD (1 = in, 0 = out)
                       0b00000001, // latchE
                       0b00000000, // directionE (1 = in, 0 = out)
                       (PORTS_PORTB_PULLUPS_ON
                       | PORTS_INT0_FALLING_EDGE
                       | PORTS_INT1_RISING_EDGE) );

    // Turn on serial transmitter and receiver. Turn on receiver interrupts
    // but leave the transmitter interrupts off (we'll turn them on when
    // there's actual data to send over the port). Make both interrupts
    // low priority, and se the bit rate to 9600bps.

    InitializeSerial (  SERIAL_TRANSMIT_DISABLE
                        | SERIAL_TRANSMIT_DISABLE_INTS
                        | SERIAL_RECEIVE_DISABLE
                        | SERIAL_RECEIVE_DISABLE_INTS
                        | SERIAL_BITRATE_9600 );

    // Turn on timer2, HP interrupts @ 1KHz
/*
    InitializeTimer2 (  TIMER2_ENABLE
                        | TIMER2_ENABLE_INTS
                        | TIMER2_1KHZ );
     Teh above code gave this error when compieling after switching to
     the new timer2.c and timer2.h 9/13/02
     Error[000] C:\FIRMWARE\IMU.C 254 : undefined identifier: TIMER2_ENABLE
     switchted to the code below.
                        */
InitializeTimer2 (TIMER2_2_5KHZ | TIMER2_ENABLE_INTS);

    // Turn on CAN: See const variable declarations above
    // for the values. Note that as soon as we initialize the
    // the bus, we can start to receive messages, so be sure
    // that interrupts are enabled VERY SOON after this call
    // (No more than about a  message length, which at top
    // speed is about 1/22KHz = 45us which at 20MHz is only
    // 250 instructions.

    InitializeCan ( CAN_BITRATE_1MBPS,
                    canMaskB0,          // Mask RXB0
                    canMaskB1,          // Mask RXB1
                    CanFilterB0F0,      // Filter RXB0 F0
                    CanFilterB0F1,      // Filter RXB0 F1
                    CanFilterB1F2,      // Filter RXB1 F2
                    CanFilterB1F3,      // Filter RXB1 F3
                    CanFilterB1F4,      // Filter RXB1 F4
                    CanFilterB1F5);     // Filter RXB1 F5


    // ALL INITIALIZATION THAT MUST BE DONE PRIOR TO INTERRUPTS
    // BEING ENABLED MUST BE DONE PRIOR TO "InitializeInterrupts()".

    InitializeInterrupts();

    // INTERRUPTS ARE INITIALIZED; FEEL FREE TO INITIALIZE THINGS
    // WHICH REQUIRE INTERRUPTS TO BE RUNNING HERE.

}



/*---------------------------------------------------------------------------\
| Function:     Parse_Can_Message
|
| Purpose:      Interpet can message and act on it.
|
| Discussion:   see purpose
|
| Expects:      a can message with directions in it
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 02/10/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

void Parse_Can_Message(CanMessage_t* receivedMsg)
{
    switch(receivedMsg->header.all)
    {
  // begin imu instruction / manipulation
        case (IMU_COMMAND):
        break;
        // IMU base sample rate (Timer2 rate)
        case (IMU_BASE_SAMPLE_RATE_CHANGE):
          switch(receivedMsg->data[0])
          {
              case 0:
                InitializeTimer2(TIMER2_OFF);
              break;
              case 1:
                InitializeTimer2(TIMER2_250HZ);
              break;
              case 2:
                InitializeTimer2(TIMER2_500HZ);
              break;
              case 3:
                InitializeTimer2(TIMER2_1KHZ);
              break;
              case 4:
                InitializeTimer2(TIMER2_2KHZ);
              break;
              case 5:
                InitializeTimer2(TIMER2_2_5KHZ);
              break;
              case 6:
                InitializeTimer2(TIMER2_5KHZ);
              break;
              case 7:
                InitializeTimer2(TIMER2_10KHZ);
              break;
              default:
                ThrowWarning(WARN_BASE_RATE_TIMER2_OUT_OF_RANGE);
              break;
          }
        break;
        // IMU ID integer divide (Sample Rate Change)
        case (IMU_INT_DIVIDE_CHANGE):
            if(receivedMsg->data[0])
            {
                //change sample rate for acc
                if(receivedMsg->data[1] )
                {
                  acc_x_sample_divisor = receivedMsg->data[1];
                  acc_y_sample_divisor = receivedMsg->data[1];
                  acc_z_sample_divisor = receivedMsg->data[1];
                  acc_q_sample_divisor = receivedMsg->data[1];
                } else {
                  acc_x_sample_divisor = 0;
                  acc_y_sample_divisor = 0;
                  acc_z_sample_divisor = 0;
                  acc_q_sample_divisor = 0;
                  acc_x_adc_result_h = 0;
                  acc_y_adc_result_h = 0;
                  acc_z_adc_result_h = 0;
                  acc_q_adc_result_h = 0;
                  acc_x_adc_result_l = 0;
                  acc_y_adc_result_l = 0;
                  acc_z_adc_result_l = 0;
                  acc_q_adc_result_l = 0;

                }
            } else {
                // change sample rete for gyro
                if(receivedMsg->data[1])
                {
                  gyro_p_sample_divisor = receivedMsg->data[1];
                  gyro_y_sample_divisor = receivedMsg->data[1];
                  gyro_r_sample_divisor = receivedMsg->data[1];
                } else {
                  gyro_p_sample_divisor = 0;
                  gyro_y_sample_divisor = 0;
                  gyro_r_sample_divisor = 0;
                  gyro_p_adc_result_h = 0;
                  gyro_p_adc_result_l = 0;
                  gyro_y_adc_result_h = 0;
                  gyro_y_adc_result_l = 0;
                  gyro_r_adc_result_h = 0;
                  gyro_r_adc_result_l = 0;

                }
            }
        break;
 // end imu instructions

 // being pressure instructions
        // pressure command off/sleep/reset
        case (PRES_COMMAND):
        // this would change the speed of timer2 for now it is
        // only implimented in the IMU_BASE_SAMPLE_RATE_CHANGE
        break;
        // Pressure ID Sample Rate Change
        case (PRES_BASE_SAMPLE_RATE_CHANGE):
            // this would change the speed of timer2 for now it is
            // only implimented in the IMU_BASE_SAMPLE_RATE_CHANGE
        break;
        // pres integer divider settings
        case (PRES_INT_DIVIDE_CHANGE):
          pres_sample_divisor = receivedMsg->data[0];
          if(! pres_sample_divisor)
          {
            pres_adc_result_l = 0;
            pres_adc_result_h = 0;
          }
        break;
 // end pressure instructions

 // begin temperature instructions
        // temperature command off/reset/sleep
        case (TEMP_COMMAND):
        break;
        // Temp ID Sample Rate Change
        case (TEMP_BASE_SAMPLE_RATE_CHANGE):
          temp_sample_divisor = receivedMsg->data[0];
          if(! temp_sample_divisor)
          {
            ADRESH = 0;
            ADRESH = 0;
          }
        break;
        // temperature integer dived settings
        case (TEMP_INT_DIVIDE_CHANGE):
        break;
// end temperature instructions
        default:
            ThrowWarning(WARN_INTEGER_DIVISOR_SENSOR_ID_BAD);
        break;
    } // end ID switch


} // end Parse_Can_Message

