/*---------------------------------------------------------------------------\
| Project:  imu LV2 PIC18F458 CAN node firmware
|
| File:     imu.h
|
| Descr.:   Demo.
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 06/01/2002 | Created.
\---------------------------------------------------------------------------*/

#ifndef _IMU_H     // Don't include more than once (last line = #endif)
#define _IMU_H

//------------------------------------------------------------------------
// Typedefs
//------------------------------------------------------------------------


enum WARNING_MESSAGES
{
    WARN_BASE_RATE_TIMER2_OUT_OF_RANGE,
    WARN_INTEGER_DIVISOR_SENSOR_ID_BAD
};


//------------------------------------------------------------------------
// #defines
//------------------------------------------------------------------------
// #define the pins of the 18F458
#define IMUTEMP_ RA0 //PIN# 19
#define CS_ RE0      //PIN 25
#define HBEN RE2     //PIN 27
#define RD_ RC0      //PIN 32
#define WR_ RC1      //PIN 35
#define TEST RC2     //PIN 36
#define INT_ RB0     //PIN 8
#define D0 RD0       //PIN 38
#define D1 RD1       //PIN 39
#define D2 RD2       //PIN 40
#define D3 RD3       //PIN 41
#define D4 RD4       //PIN 2
#define D5 RD5       //PIN 3
#define D6 RD6       //PIN 4
#define D7 RD7       //PIN 5
#define CANRX RB3    //PIN 11
#define CANTX RB2    //PIN 10

// some defines for types of can messages
// 0b0001110110000010
// bits 0-3 #of bytes in message
// bit    4 RTR
// bits 5-6 encoding type (not currently used = 00)
// bits 7-9 message identifier (type of message int divide change, data,...)
// bits 10-15 node id (i.e. IMU, Press, Temp)

#define IMU_COMMAND 0b0001000000000001
#define IMU_BASE_SAMPLE_RATE_CHANGE 0b0001000010000001
#define IMU_INT_DIVIDE_CHANGE 0b0001000100000010
#define IMU_ACC_DATA_OUT 0b0001001000001000
#define IMU_GYRO_DATA_OUT 0b0001001010000110

#define PRES_COMMAND 0b0001110000000001
#define PRES_BASE_SAMPLE_RATE_CHANGE 0b0001110010000001
#define PRES_INT_DIVIDE_CHANGE 0b0001110100000001
#define PRES_DATA_OUT 0b0001110110000010

#define TEMP_COMMAND 0b0101000000000001
#define TEMP_BASE_SAMPLE_RATE_CHANGE 0b0101000010000001
#define TEMP_INT_DIVIDE_CHANGE 0b0101000100000001
#define TEMP_DATA_OUT 0b0101000110000010

//------------------------------------------------------------------------
// Function declarations
//------------------------------------------------------------------------
void Initialize (void);
void Parse_Can_Message(CanMessage_t* receivedMsg);


// None


//------------------------------------------------------------------------
// Global variable declarations
//------------------------------------------------------------------------
//These globals are used to define the sample rates for IMU and Temp and Pressure
extern uint8_t acc_x_sample_divisor; //Default rate of 2.5kHz (w/timer2@2.5kHz)
extern uint8_t acc_y_sample_divisor; //Default rate of 2.5kHz (w/timer2@2.5kHz)
extern uint8_t acc_z_sample_divisor; //Default rate of 2.5kHz (w/timer2@2.5kHz)
extern uint8_t acc_q_sample_divisor; //Default rate of 2.5kHz (w/timer2@2.5kHz)
extern uint8_t gyro_p_sample_divisor;//Default rate of 833Hz (w/timer2@2.5kHz)
extern uint8_t gyro_y_sample_divisor;//Default rate of 833Hz (w/timer2@2.5kHz)
extern uint8_t gyro_r_sample_divisor;//Default rate of 833Hz (w/timer2@2.5kHz)
extern uint8_t pres_sample_divisor;//Default rate of 100Hz (w/timer2@2.5kHz)
extern uint8_t temp_sample_divisor;//Default rate of 10Hz (w/timer2@2.5kHz)

// if a sample needs to be taken its associated flag will be set to TRUE
extern bit acc_x_flag;
extern bit acc_y_flag;
extern bit acc_z_flag;
extern bit acc_q_flag;
extern bit gyro_p_flag;
extern bit gyro_y_flag;
extern bit gyro_r_flag;
extern bit pres_flag;
extern bit temp_flag;

// holds commands for Max197 AD chip, see data sheet for more details
extern uint8_t max197_cmd;

// hold results from the ADC before sending in the CAN message
// not adc_result for temp cause we get it directly from pic reg ADRESL/H
extern uint8_t acc_x_adc_result_h;
extern uint8_t acc_y_adc_result_h;
extern uint8_t acc_z_adc_result_h;
extern uint8_t acc_q_adc_result_h;

extern uint8_t gyro_p_adc_result_h;
extern uint8_t gyro_y_adc_result_h;
extern uint8_t gyro_r_adc_result_h;

extern uint8_t acc_x_adc_result_l;
extern uint8_t acc_y_adc_result_l;
extern uint8_t acc_z_adc_result_l;
extern uint8_t acc_q_adc_result_l;

extern uint8_t gyro_p_adc_result_l;
extern uint8_t gyro_y_adc_result_l;
extern uint8_t gyro_r_adc_result_l;

extern uint8_t pres_adc_result_l;
extern uint8_t pres_adc_result_h;

// flags for when a can message is ready to be sent
extern bit acc_go;
extern bit gyro_go;
extern bit pres_go;
extern bit temp_go;



//------------------------------------------------------------------------
// Module-wide variable declarations
//------------------------------------------------------------------------

// None


#endif              // _IMU_H


